"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cluster_1 = __importDefault(require("./commands/cluster"));
const cluster_slots_1 = __importDefault(require("./cluster-slots"));
class RedisCluster {
    static defineCommand(on, name, command) {
        on[name] = async function (...args) {
            const transformedArguments = command.transformArguments(...args);
            return command.transformReply(await this.sendCommand(transformedArguments, command.FIRST_KEY_INDEX, command.IS_READ_ONLY));
        };
    }
    static create(options) {
        return new RedisCluster(options);
    }
    #options;
    #slots;
    constructor(options) {
        this.#options = options;
        this.#slots = new cluster_slots_1.default(options);
    }
    async connect() {
        return this.#slots.connect();
    }
    async sendCommand(args, firstKeyIndex, isReadonly, redirections = 0) {
        const firstKey = firstKeyIndex ? args[firstKeyIndex] : undefined, client = this.#slots.getClient(firstKey, isReadonly);
        try {
            return await client.sendCommand(args);
        }
        catch (err) {
            if (err.message.startsWith('ASK')) {
                // TODO
            }
            else if (err.message.startsWith('MOVED')) {
                await this.#slots.discover(client);
                if (redirections < (this.#options.maxCommandRedirections ?? 16)) {
                    return this.sendCommand(args, firstKeyIndex, isReadonly, redirections + 1);
                }
            }
            throw err;
        }
    }
    disconnect() {
        return this.#slots.disconnect();
    }
}
exports.default = RedisCluster;
for (const [name, command] of Object.entries(cluster_1.default)) {
    RedisCluster.defineCommand(RedisCluster.prototype, name, command);
}
