"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.itWithClient = exports.TEST_REDIS_SERVERS = exports.TestRedisServers = void 0;
const strict_1 = __importDefault(require("assert/strict"));
const client_1 = __importDefault(require("./client"));
const child_process_1 = require("child_process");
const events_1 = require("events");
const tcp_port_used_1 = __importDefault(require("tcp-port-used"));
const which_1 = __importDefault(require("which"));
var TestRedisServers;
(function (TestRedisServers) {
    TestRedisServers[TestRedisServers["OPEN"] = 0] = "OPEN";
    TestRedisServers[TestRedisServers["PASSWORD"] = 1] = "PASSWORD";
})(TestRedisServers = exports.TestRedisServers || (exports.TestRedisServers = {}));
exports.TEST_REDIS_SERVERS = {};
before(() => {
    return Promise.all([
        spawnOpenServer(),
        spawnPasswordServer()
    ]);
});
async function spawnOpenServer() {
    exports.TEST_REDIS_SERVERS[TestRedisServers.OPEN] = {
        port: await spawnRedisServer()
    };
}
async function spawnPasswordServer() {
    exports.TEST_REDIS_SERVERS[TestRedisServers.PASSWORD] = {
        port: await spawnRedisServer(['--requirepass', 'password']),
        username: 'default',
        password: 'password'
    };
}
function itWithClient(type, title, fn) {
    it(title, async () => {
        const client = client_1.default.create({
            socket: exports.TEST_REDIS_SERVERS[type]
        });
        await client.connect();
        try {
            await client.flushAll();
            await fn(client);
        }
        finally {
            await client.flushAll();
            await client.disconnect();
        }
    });
}
exports.itWithClient = itWithClient;
const REDIS_PATH = which_1.default.sync('redis-server');
let port = 6379;
async function spawnRedisServer(args) {
    const currentPort = port++, process = child_process_1.spawn(REDIS_PATH, [
        '--save',
        '',
        '--port',
        currentPort.toString(),
        ...(args ?? [])
    ]);
    // TODO: catch process exit
    await tcp_port_used_1.default.waitForStatus(currentPort, '127.0.0.1', true, 10, 1000);
    after(() => {
        strict_1.default.ok(process.kill());
        return events_1.once(process, 'close');
    });
    return currentPort;
}
